@@ -8,6 +8,7 @@
 #include "strutl.h"
 
 #include <dlfcn.h>
+#include <sys/wait.h>
 
 #define CHECKARGC(pred) \
 ({\
@@ -57,8 +58,7 @@ command_input(struct confmodule *mod, char *arg)
     else
         mod->frontend->methods.add_noninteractive(mod->frontend, q);
 
-    if (q->priority != NULL)
-        free(q->priority);
+    free(q->priority);
     q->priority = strdup(priority);
 
     if (visible) 
@@ -131,9 +131,11 @@ command_capb(struct confmodule *mod, char *arg)
             mod->frontend->capability |= DCF_CAPB_PROGRESSCANCEL;
         else if (strcmp(argv[i], "align") == 0)
             mod->frontend->capability |= DCF_CAPB_ALIGN;
+        else if (strcmp(argv[i], "escape") == 0)
+            mod->frontend->capability |= DCF_CAPB_ESCAPE;
     }
 
-    if (asprintf(&out, "%u multiselect backup progresscancel align",
+    if (asprintf(&out, "%u multiselect backup progresscancel align escape",
                  CMDSTATUS_SUCCESS) == -1)
         DIE("Out of memory");
 
@@ -258,6 +260,33 @@ command_go(struct confmodule *mod, char *arg)
     return out;
 }
 
+static char *escapestr_frontend(const char *in)
+{
+	static size_t buflen = 0;
+	static char *buf = NULL;
+	size_t inlen;
+	const char *p;
+
+	if (in == 0) return 0;
+
+	inlen = strlen(in) + 1;
+
+	/* Each newline will consume an additional byte due to escaping. */
+	for (p = in; *p; ++p)
+		if (*p == '\\' || *p == '\n')
+			++inlen;
+
+	if (buflen < inlen) {
+		buflen = inlen;
+		buf = realloc(buf, buflen * sizeof *buf);
+		if (!buf)
+			DIE("Out of memory");
+	}
+
+	strescape(in, buf, buflen, STRESCAPE_CAPB);
+	return buf;
+}
+
 char *
 command_get(struct confmodule *mod, char *arg)
 {
@@ -276,8 +305,14 @@ command_get(struct confmodule *mod, char *arg)
     else 
     {
         const char *value = question_getvalue(q, "C");
-        asprintf(&out, "%u %s",
-                CMDSTATUS_SUCCESS, value ? value : "");
+        if (mod->frontend->capability & DCF_CAPB_ESCAPE)
+        {
+            char *esc = escapestr_frontend(value);
+            asprintf(&out, "%u %s", CMDSTATUS_ESCAPEDDATA, esc ? esc : "");
+        }
+        else
+            asprintf(&out, "%u %s",
+                    CMDSTATUS_SUCCESS, value ? value : "");
     }
     question_deref(q);
 
@@ -465,7 +500,15 @@ command_metaget(struct confmodule *mod, char *arg)
     if (value == NULL)
         asprintf(&out, "%u %s does not exist", CMDSTATUS_BADQUESTION, argv[1]);
     else
-        asprintf(&out, "%u %s", CMDSTATUS_SUCCESS, value);
+    {
+        if (mod->frontend->capability & DCF_CAPB_ESCAPE)
+        {
+            char *esc = escapestr_frontend(value);
+            asprintf(&out, "%u %s", CMDSTATUS_ESCAPEDDATA, esc);
+        }
+        else
+            asprintf(&out, "%u %s", CMDSTATUS_SUCCESS, value);
+    }
     free(value);
 
     question_deref(q);
@@ -575,6 +618,12 @@ command_stop(struct confmodule *mod, char *arg)
 
     argc = strcmdsplit(arg, argv, DIM(argv));
     CHECKARGC(== 0);
+	
+    if (mod->pid != 0)
+    {
+        waitpid(mod->pid, NULL, 0);
+    }
+
     return strdup("");
 }
 
