@@ -36,6 +36,7 @@ struct option g_dpc_args[] = {
 	{ "all", 0, NULL, 'a' },
 	{ "unseen-only", 0, NULL, 'u' },
 	{ "force", 0, NULL, 'F' },
+	{ "no-reload", 0, NULL, 'n' },
 	{ 0, 0, 0, 0 }
 };
 
@@ -43,7 +44,7 @@ static struct configuration *g_config;
 static struct frontend *g_frontend;
 static struct template_db *g_templates;
 static struct question_db *g_questions;
-static int opt_all, opt_force;
+static int opt_all = 0, opt_force = 0, no_reload = 0;
 
 /************************************************************************
  * Function: cleanup
@@ -91,8 +92,15 @@ static void sighandler(int sig)
  ************************************************************************/
 static void usage(void)
 {
-	printf("dpkg-reconfigure [--frontend <frontend>] [--priority <priority>] package\n");
-	printf("dpkg-reconfigure [-f <frontend>] [-p <priority>] package\n");
+	printf("Usage: dpkg-reconfigure [options] packages\n"
+		"  -f, --frontend <frontend>	Select the frontend to use.\n"
+		"  -p, --priority <priority>	Specify the minimum priority.\n"
+		"      --default-priority	Use default priority.\n"
+		"  -a, --all			Reconfigure all packages.\n"
+		"  -u, --unseen-only		Show only not yet seen questions.\n"
+		"      --force			Force reconfiguration of broken packages.\n"
+		"      --no-reload		Do not reload temlates.\n"
+		"  -h, --help			Display this help.\n");
 	exit(0);
 }
 
@@ -178,6 +186,64 @@ static char *getfield(const char *package, const char *fieldname)
 	return field;
 }
 
+/************************************************************************
+ * Function: getallpackages
+ * Inputs: none
+ * Outputs: char * - space separated list of packages
+ * Description: gets the list of installed packages
+ ************************************************************************/
+static char *getallpackages()
+{
+	char *buf = NULL;
+	int pipefd[2];
+	int pid;
+
+	if (pipe(pipefd) < 0)
+	{
+		perror("pipe");
+		return NULL;
+	}
+
+	pid = fork();
+	if (pid < 0)
+	{
+		perror("fork");
+		close(pipefd[0]);
+		close(pipefd[1]);
+		return NULL;
+	}
+	else if (pid == 0) /* child */
+	{
+		close(pipefd[0]);
+		dup2(pipefd[1], 1);
+		close(pipefd[1]);
+		execlp("dpkg-query", "dpkg-query", "--show", "-f=${Package} ", NULL);
+		/* if we reach here, then execlp failed */
+		perror("execlp");
+		_exit (127);
+	}
+	else /* parent */
+	{
+		FILE *fp;
+		size_t size = 0, offset = 0;
+
+		close(pipefd[1]);
+		fp = fdopen(pipefd[0], "r");
+
+		while (!feof(fp))
+		{
+			size += 2048;
+			buf = realloc(buf, size);
+			if (!buf)
+				exit(1);
+			fread(buf+offset,sizeof(*buf), 2048, fp);
+			offset = size;
+		}
+		fclose(fp);
+	}
+	return buf;
+}
+
 /************************************************************************
  * Function: is_confmodule
  * Inputs: filename - filename to check
@@ -288,6 +354,8 @@ static int runscript (const char *pkg, const char *script, const char *param, co
 	char *filename;
 	char *argv[5] = {0};
 
+	setenv("DPKG_MAINTSCRIPT_NAME", script, 1);
+
 	filename = control_path(pkg, script);
 	if (! file_exists(filename,S_IXUSR|S_IXGRP|S_IXOTH))
 	{
@@ -331,51 +399,41 @@ static int runscript (const char *pkg, const char *script, const char *param, co
 	return ret;
 }
 
-/************************************************************************
- * Function: reconfigure
- * Inputs: pkgs - pointer to an array of packages
- *         i - first index, max - max index
- * Outputs: int - 0 if no error, >0 otherwise
- * Description: Configure each of the packages passed in
- * Assumptions: none
- ************************************************************************/
-static int reconfigure(char **pkgs, int i, int max)
+static int reconfigure(char *pkg)
 {
-	int ret = 0;
+	int ret;
 	char *filename;
-	char *pkg;
 	char *version = NULL;
 
-	while (i < max)
+	g_frontend->methods.set_title(g_frontend, pkg);
+	if (!opt_force && strstr(getfield(pkg, STATUSFIELD), " ok installed") == 0)
+		DIE("%s is not fully installed", pkg);
+
+	if (!no_reload)
 	{
-		pkg = pkgs[i++];
-
-		g_frontend->methods.set_title(g_frontend, pkg);
-		if (strstr(getfield(pkg, STATUSFIELD), " ok installed") == 0)
-			DIE("%s is not fully installed", pkg);
-
 		filename = control_path(pkg, "templates");
 		if (file_exists(filename, S_IRUSR|S_IRGRP|S_IROTH))
 			template_db_loadfile(g_templates, g_questions, filename, pkg, DC_LOADTEMPLATE_NONE);
 		free(filename);
-
-		version = getfield(pkg, VERSIONFIELD);
-		
-		/* Simulation of reinstalling a package, without bothering with
-		   removing the files and putting them back. Just like in a
-		   regular reinstall, run config, and postinst scripts in
-		   sequence, with args. Do not run postrm, because the postrm
-		   can depend on the package's files actually being gone
-		   already. */
-		if ((ret = runscript(pkg, "prerm", "upgrade", version)) != 0)
-			break;
-		if ((ret = runscript(pkg, "config", "reconfigure", version)) != 0)
-			break;
-		if ((ret = runscript(pkg, "postinst", "configure", version)) != 0)
-			break;
-
-		i++;
 	}
+
+	version = getfield(pkg, VERSIONFIELD);
+	
+	setenv("DPKG_MAINTSCRIPT_PACKAGE", pkg, 1);
+	setenv("DPKG_MAINTSCRIPT_VERSION", version, 1);
+
+	/* Simulation of reinstalling a package, without bothering with
+	   removing the files and putting them back. Just like in a
+	   regular reinstall, run config, and postinst scripts in
+	   sequence, with args. Do not run postrm, because the postrm
+	   can depend on the package's files actually being gone
+	   already. */
+	ret = runscript(pkg, "prerm", "upgrade", version);
+	if (ret == 0)
+		ret = runscript(pkg, "config", "reconfigure", version);
+	if (ret == 0)
+		ret = runscript(pkg, "postinst", "configure", version);
+
 	return ret;
 }
 
@@ -388,7 +446,7 @@ static int reconfigure(char **pkgs, int i, int max)
  ************************************************************************/
 int main(int argc, char **argv)
 {
-	int opt, ret;
+	int opt, ret = 0;
 	int unseen_only=0;
 	int default_priority=0;
 	char *priority_override=NULL;
@@ -401,7 +459,7 @@ int main(int argc, char **argv)
 
 	g_config = config_new();
 
-	while ((opt = getopt_long(argc, argv, "hf:p:duaF", g_dpc_args, NULL)) > 0)
+	while ((opt = getopt_long(argc, argv, "hf:p:duaFn", g_dpc_args, NULL)) > 0)
 	{
 		switch (opt)
 		{
@@ -412,9 +470,10 @@ int main(int argc, char **argv)
 		case 'u': unseen_only=1; break;
 		case 'a': opt_all = 1; break;
 		case 'F': opt_force = 1; break;
+		case 'n': no_reload = 1; break;
 		}
 	}
-	if (optind == argc) {
+	if (!opt_all && optind == argc) {
 		fprintf(stderr, "please specify a package to reconfigure\n");
 		exit(1);
 	}
@@ -446,7 +505,23 @@ int main(int argc, char **argv)
 		DIE("Cannot initialize DebConf frontend");
 
 	setenv("DEBIAN_HAS_FRONTEND", "1", 1);
-	ret = reconfigure(argv, optind, argc);
+	setenv("DEBCONF_RECONFIGURE", "1", 1);
+	if (opt_all)
+	{
+		char *packages = getallpackages();
+		char *pkg = strtok(packages, " ");
+		while (pkg != NULL && ret == 0)
+		{
+			ret = reconfigure(pkg);
+			pkg = strtok(NULL, " ");
+		}
+	}
+	else
+	{
+		int i;
+		for (i = optind; i < argc && ret == 0; i++)
+			ret = reconfigure(argv[i]);
+	}
 	/* shutting down .... sync the database and shutdown the modules */
 	cleanup();
 	return ret;
