@@ -163,6 +163,51 @@ static unsigned int parse_flags(char *string)
     return ret;
 }
 
+static FILE *rfc822db_file_open(struct configuration *config, const char *configpath, int *retval)
+{
+    const char *path;
+    FILE *inf = NULL;
+    char tmp[1024];
+
+    *retval = DC_OK;
+    
+    snprintf(tmp, sizeof(tmp), "%s::path", configpath);
+    path = config->get(config, tmp, 0);
+    if (path == NULL)
+    {
+        INFO(INFO_VERBOSE, "Cannot open database <empty>");
+        *retval = DC_NOTOK;
+    }
+    else if ((inf = fopen(path, "r")) == NULL)
+    {
+        if (errno == ENOENT)
+        {
+            const char *modestr;
+            int mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
+
+            INFO(INFO_DEBUG, "Database file doesn't exist. Trying to create it");
+
+            snprintf(tmp, sizeof(tmp), "%s::mode", configpath);
+            modestr = config->get(config, tmp, 0);
+            if (modestr)
+                mode = strtol(modestr, NULL, 8);
+
+            outf = fopen(path, "w");
+            if ((outf == NULL) || (fclose(outf) != 0) || (chmod(path, mode) != 0))
+            {
+                INFO(INFO_VERBOSE, "Cannot create database file %s: %s",
+                     path, strerror(errno));
+                *retval = DC_NOTOK;
+            }
+            outf = NULL;
+        }
+        else
+            *retval = DC_NOTOK;
+    }
+
+    return inf;
+}
+
 void rfc822db_template_destroyitem(void *data)
 {
     template_deref((struct template *) data);
@@ -209,20 +254,15 @@ static int rfc822db_template_shutdown(struct template_db *db)
 static int rfc822db_template_load(struct template_db *db)
 {
     struct template_db_cache *dbdata = db->data;
-    char tmp[1024];
-    const char *path;
     FILE *inf;
     struct rfc822_header *header = NULL;
+    int openerror;
 
     INFO(INFO_VERBOSE, "rfc822db_template_load(db)");
-    snprintf(tmp, sizeof(tmp), "%s::path", db->configpath);
-    path = db->config->get(db->config, tmp, 0);
-    if (path == NULL ||
-        (inf = fopen(path, "r")) == NULL)
+    inf = rfc822db_file_open(db->config, db->configpath, &openerror);
+    if (!inf)
     {
-        INFO(INFO_VERBOSE, "Cannot open template file %s",
-            path ? path : "<empty>");
-        return DC_NOTOK;
+        return openerror;
     }
 
     while ((header = rfc822_parse_stanza(inf)) != NULL)
@@ -337,12 +377,12 @@ void rfc822db_template_dump(const void *node, const VISIT which, const int depth
         break;
     case postorder: 
     case leaf:
-        p = template_lget((struct template *) t, NULL, "tag");
+        p = template_lget(t, NULL, "tag");
         INFO(INFO_VERBOSE, "dumping template %s", p);
 
         for (field = template_fields_list; *field != NULL; field++)
         {
-            p = template_lget((struct template *) t, NULL, *field);
+            p = template_lget(t, NULL, *field);
             if (p != NULL)
             {
                 if (strcmp(*field, "tag") == 0)
@@ -353,13 +393,13 @@ void rfc822db_template_dump(const void *node, const VISIT which, const int depth
             }
         }
     
-        lang = template_next_lang((struct template *) t, NULL);
+        lang = template_next_lang(t, NULL);
         while (lang) 
         {
             for (field = template_fields_list; *field != NULL; field++)
             {
-                p = template_lget((struct template *) t, lang, *field);
-                if (p != NULL && p != template_lget((struct template *) t, NULL, *field))
+                p = template_lget(t, lang, *field);
+                if (p != NULL && p != template_lget(t, NULL, *field))
                 {
                     if (strcmp(lang, "C") == 0)
                         fprintf(outf, "%c%s-C: %s\n",
@@ -369,7 +409,7 @@ void rfc822db_template_dump(const void *node, const VISIT which, const int depth
                             toupper((*field)[0]), (*field)+1, lang, escapestr(p));
                 }
             }
-            lang = template_next_lang((struct template *) t, lang);
+            lang = template_next_lang(t, lang);
         }
         fprintf(outf, "\n");
     }
@@ -565,23 +605,15 @@ static int rfc822db_question_shutdown(struct question_db *db)
 static int rfc822db_question_load(struct question_db *db)
 {
     struct question_db_cache *dbdata = db->data;
-    char tmp[1024];
-    const char *path;
     FILE *inf;
     struct rfc822_header *header = NULL;
+    int openerror;
 
     INFO(INFO_VERBOSE, "rfc822db_question_load(db)");
-    snprintf(tmp, sizeof(tmp), "%s::path", db->configpath);
-    path = db->config->get(db->config, tmp, 0);
-    if (path == NULL ||
-        (inf = fopen(path, "r")) == NULL)
+    inf = rfc822db_file_open(db->config, db->configpath, &openerror);
+    if (!inf)
     {
-        if (errno == ENOENT)
-            return DC_NOTOK;
-
-        INFO(INFO_VERBOSE, "Cannot open config database %s: %s",
-            path ? path : "<empty>", strerror(errno));
-        return DC_NOTOK;
+        return openerror;
     }
 
     while ((header = rfc822_parse_stanza(inf)) != NULL)
