@@ -94,6 +94,7 @@ static gboolean handle_exposed_banner(GtkWidget * widget,
 {
     struct frontend_data * fe_data = fe->data;
     GdkScreen * screen;
+    GdkWindow * window;
     PangoFontDescription * font;
     PangoLayout * layout;
     gint text_width;
@@ -112,8 +113,9 @@ static gboolean handle_exposed_banner(GtkWidget * widget,
         pango_layout_set_font_description(layout, font);
         pango_layout_get_pixel_size(layout, &text_width, &text_height);
         screen = gtk_window_get_screen(GTK_WINDOW(fe_data->window));
+        window = gtk_widget_get_window(widget);
         /* XXX: damn magic numbers */
-        gdk_draw_layout(widget->window, gdk_gc_new(widget->window),
+        gdk_draw_layout(window, gdk_gc_new(window),
             gdk_screen_get_width(screen) - text_width - 4
             - DEFAULT_PADDING * 2, 4, layout);
         g_object_unref(layout);
