@@ -205,8 +205,16 @@ show_help (struct frontend *obj, struct question *q)
 	free(descr);
 }
 
+struct choices {
+	int count;
+	char **choices;
+	char **choices_translated;
+	char *selected;
+	int *tindex;
+};
+
 static void
-printlist (struct frontend *obj, struct question *q, int count, char **choices_translated, int *tindex, char *selected)
+printlist (struct frontend *obj, struct question *q, const struct choices *choices)
 {
 	int choice_min = -1;
 	int num_cols, num_lines;
@@ -217,35 +225,36 @@ printlist (struct frontend *obj, struct question *q, int count, char **choices_t
 	char **output;
 	int line = 0, max_len = 0, col = 0;
 	int width = getwidth();
-	char **fchoices = malloc(sizeof(char *) * count);
-	i = count;
+	char **fchoices = malloc(sizeof(char *) * choices->count);
+	i = choices->count;
 	do {
 		i /= 10;
 		logcount++;
 	} while (i > 0);
 
 	if (obj->methods.can_align(obj, q)) {
-		stralign(choices_translated, count);
+		stralign(choices->choices_translated, choices->count);
 		/* Display only one column to get meaningful alignment. */
 		width = 1;
 	}
 
 	/*  Set string arrays  */
-	for (i=0; i < count; i++)
+	for (i=0; i < choices->count; i++)
 	{
 		/*  Trailing spaces are a placeholder to add [*] for
-		    selected values */
-		asprintf(&(fchoices[i]), "%3d. %s    ", i+1, choices_translated[i]);
-		if (selected[tindex[i]])
-			strcpy(fchoices[i]+strlen(fchoices[i])-3, "[*]");
+		    selected values
+		    Comma is needed for proper speech synthesis pause */
+		asprintf(&(fchoices[i]), "%3d: %s,    ", i+1, choices->choices_translated[i]);
+		if (choices->selected[choices->tindex[i]])
+			strcpy(fchoices[i]+strlen(fchoices[i])-5, " [*],");
 		if (strwidth(fchoices[i]) < choice_min || choice_min == -1)
 			choice_min = strwidth(fchoices[i]);
 		if (strwidth(fchoices[i]) > width)
 			width = strwidth(fchoices[i]);
 	}
 	num_cols = width / choice_min;
-	if (num_cols > count)
-		num_cols = count;
+	if (num_cols > choices->count)
+		num_cols = choices->count;
 	col_width = malloc (sizeof(int) * num_cols);
 	num_cols++;
 	while (1)
@@ -254,10 +263,10 @@ printlist (struct frontend *obj, struct question *q, int count, char **choices_t
 		num_cols--;
 		if (num_cols == 0)
 			break;
-		num_lines = (count - 1) / num_cols + 1;
+		num_lines = (choices->count - 1) / num_cols + 1;
 		for (i = 0; i < num_cols; i++)
 			col_width[i] = 0;
-		for (i = 0; i < count; i++)
+		for (i = 0; i < choices->count; i++)
 		{
 			int current_col = i / num_lines;
 			if (strwidth(fchoices[i]) > col_width[current_col])
@@ -274,7 +283,7 @@ printlist (struct frontend *obj, struct question *q, int count, char **choices_t
 	}
 	if (num_cols == 0)
 	{
-		num_lines = count;
+		num_lines = choices->count;
 		num_cols = 1;
 	}
 	output = malloc(sizeof(char *) * num_lines);
@@ -283,7 +292,7 @@ printlist (struct frontend *obj, struct question *q, int count, char **choices_t
 		output[i] = malloc(MB_LEN_MAX * width + 1);
 		*(output[i]) = '\0';
 	}
-	for (i = 0; i < count; i++)
+	for (i = 0; i < choices->count; i++)
 	{
 		strcat(output[line], fchoices[i]);
 		if (strwidth(output[line]) > max_len)
@@ -309,7 +318,7 @@ printlist (struct frontend *obj, struct question *q, int count, char **choices_t
 	}
 	free(output);
 	free(col_width);
-	for (i = 0; i < count; i++)
+	for (i = 0; i < choices->count; i++)
 		free(fchoices[i]);
 	free(fchoices);
 }
@@ -339,8 +348,8 @@ static int text_handler_boolean(struct frontend *obj, struct question *q)
 	}
 
 	do {
-		printf("  %d. %s%s", 1, question_get_text(obj, "debconf/yes", "Yes"), (1 == def ? " [*]" : "    "));
-		printf("  %d. %s%s", 2, question_get_text(obj, "debconf/no", "No"), (2 == def ? " [*]" : ""));
+		printf("  %d: %s%s", 1, question_get_text(obj, "debconf/yes", "Yes"), (1 == def ? " [*]" : "    "));
+		printf("  %d: %s%s", 2, question_get_text(obj, "debconf/no", "No"), (2 == def ? " [*]" : ""));
 		printf("\n");
 		if (def)
 			printf(question_get_text(obj, "debconf/text-prompt-default",
@@ -366,6 +375,82 @@ static int text_handler_boolean(struct frontend *obj, struct question *q)
 	return DC_OK;
 }
 
+/*
+ * Function: choices_delete
+ * Input: struct choices *c - choices values, translations, indices, and select
+ * Description: frees the structure and its content
+ * Assumptions: none
+ */
+static void choices_delete(struct choices *c)
+{
+	int i;
+	if (c)
+	{
+		if (c->choices)
+		{
+			for (i = 0; i < c->count; i++)
+				free(c->choices[i]);
+			free(c->choices);
+		}
+		if (c->choices_translated)
+		{
+			for (i = 0; i < c->count; i++)
+				free(c->choices_translated[i]);
+			free(c->choices_translated);
+		}
+		free(c->selected);
+		free(c->tindex);
+		free(c);
+	}
+}
+
+/*
+ * Function: choices_get
+ * Input: struct frontend *obj - frontend object
+ *        struct question *q - question to ask
+ * Output: struct choices * - choices values, translations, indices, and select
+ * Description: retrieve question choices, translations, indices
+ * Assumptions: none
+ */
+static struct choices *choices_get(struct frontend *obj, struct question *q)
+{
+	struct choices *retval = NULL;
+	int i, count;
+	char *choices_vals = q_get_choices_vals(obj, q);
+
+	count = strgetargc(choices_vals);
+	if (count > 0)
+	{
+		int dcount;
+		retval = (struct choices *)malloc(sizeof(struct choices));
+		retval->count = count;
+		retval->choices = malloc(sizeof(char *) * count);
+		retval->choices_translated = malloc(sizeof(char *) * count);
+		for (i = 0; i < count; ++i)
+		{
+			retval->choices[i] = retval->choices_translated[i] = NULL;
+		}
+		retval->tindex = malloc(sizeof(int) * count);
+		retval->selected = calloc(1, sizeof(char) * retval->count);
+		
+		char *indices = q_get_indices(obj, q);
+		char *choices = q_get_choices(obj, q);
+		dcount = strchoicesplitsort(choices_vals, choices, indices, retval->choices, retval->choices_translated, retval->tindex, count);
+		free(choices);
+		free(indices);
+
+		if (dcount != count)
+		{
+			choices_delete(retval);
+			retval = NULL;
+		}
+	}
+
+	free(choices_vals);
+
+	return retval;
+}
+
 /*
  * Function: text_handler_multiselect
  * Input: struct frontend *obj - frontend object
@@ -373,53 +458,38 @@ static int text_handler_boolean(struct frontend *obj, struct question *q)
  * Output: int - DC_OK, DC_NOTOK
  * Description: handler for the multiselect question type
  * Assumptions: none
- *
- * TODO: factor common code with select
  */
 static int text_handler_multiselect(struct frontend *obj, struct question *q)
 {
-	char **choices;
-	char **choices_translated;
+	struct choices *choices = NULL;
 	char **defaults;
-	char *selected, *defval, *cp;
+	char *defval, *cp;
 	char answer[4096] = {0};
-	int i, j, count = 0, dcount, choice;
-	int *tindex = NULL;
+	int i, j, dcount, choice;
 	int ret = DC_OK;
-	const char *indices = q_get_indices(obj, q);
 
-	count = strgetargc(q_get_choices_vals(obj, q));
-	if (count <= 0)
+	choices = choices_get(obj, q);
+	if (choices == NULL)
 		return DC_NOTOK;
-	choices = malloc(sizeof(char *) * count);
-	choices_translated = malloc(sizeof(char *) * count);
-	for (i = 0; i < count; ++i)
-	{
-		choices[i] = choices_translated[i] = NULL;
-	}
-	tindex = malloc(sizeof(int) * count);
-	if (strchoicesplitsort(q_get_choices_vals(obj, q), q_get_choices(obj, q), indices, choices, choices_translated, tindex, count) != count)
-	{
-		ret = DC_NOTOK;
-		goto CleanUp_TINDEX;
-	}
 
-	defaults = malloc(sizeof(char *) * count);
-	dcount = strchoicesplit(question_getvalue(q, ""), defaults, count);
-	selected = calloc(1, sizeof(char) * count);
+	defaults = malloc(sizeof(char *) * choices->count);
+	dcount = strchoicesplit(question_getvalue(q, ""), defaults, choices->count);
 
-	for (j = 0; j < dcount; j++)
-		for (i = 0; i < count; i++) {
-			if (strcmp(choices[tindex[i]], defaults[j]) == 0)
-				selected[tindex[i]] = 1;
+	for (j = 0; j < dcount; j++) {
+		for (i = 0; i < choices->count; i++) {
+			if (strcmp(choices->choices[choices->tindex[i]], defaults[j]) == 0)
+				choices->selected[choices->tindex[i]] = 1;
 		}
-
+		free(defaults[j]);
+	}
+	free(defaults);
+	
 	i = 0;
 
-	defval = malloc(10*count);
+	defval = malloc(10*choices->count);
 	*defval = '\0';
-	for (i = 0; i < count; i++)
-		if (selected[tindex[i]])
+	for (i = 0; i < choices->count; i++)
+		if (choices->selected[choices->tindex[i]])
 		{
 			char buf[10];
 			if (*defval != '\0')
@@ -429,7 +499,7 @@ static int text_handler_multiselect(struct frontend *obj, struct question *q)
 		}
 
   DISPLAY:
-	printlist (obj, q, count, choices_translated, tindex, selected);
+	printlist (obj, q, choices);
 	printf(question_get_text(obj, "debconf/text-prompt-default-string", 
 		"Prompt: '%c' for help, default=%s> "), CHAR_HELP, defval);
 	get_answer(answer, sizeof(answer));
@@ -440,8 +510,8 @@ static int text_handler_multiselect(struct frontend *obj, struct question *q)
 	}
 	else if (answer[0] == CHAR_CLEAR && answer[1] == 0)
 	{
-		for (i = 0; i < count; i++)
-			selected[i] = 0;
+		for (i = 0; i < choices->count; i++)
+			choices->selected[i] = 0;
 	}
 	else if (obj->methods.can_go_back (obj, q) &&
 	         answer[0] == CHAR_GOBACK && answer[1] == 0)
@@ -452,43 +522,31 @@ static int text_handler_multiselect(struct frontend *obj, struct question *q)
 
 	if (!(ISEMPTY(answer)))
 	{
-		for (i = 0; i < count; i++)
-			selected[i] = 0;
+		for (i = 0; i < choices->count; i++)
+			choices->selected[i] = 0;
 		for (cp = answer; cp != NULL && *cp != '\0'; cp = strchr(cp+1, ' '))
 		{
 			choice = atoi(cp) - 1;
-			if (choice >= 0 && choice < count)
-				selected[tindex[choice]] = 1;
+			if (choice >= 0 && choice < choices->count)
+				choices->selected[choices->tindex[choice]] = 1;
 		}
 	}
 
 	answer[0] = 0;
-	for (i = 0; i < count; i++)
+	for (i = 0; i < choices->count; i++)
 	{
-		if (selected[i])
+		if (choices->selected[i])
 		{
 			if (answer[0] != 0)
 				strvacat(answer, sizeof(answer), ", ", NULL);
-			strvacat(answer, sizeof(answer), choices[i], NULL);
+			strvacat(answer, sizeof(answer), choices->choices[i], NULL);
 		}
 	}
 	question_setvalue(q, answer);
 
   CleanUp_DEFVAL:
 	free(defval);
-	free(selected);
-	for (i = 0; i < dcount; i++)
-		free(defaults[i]);
-	free(defaults);
-  CleanUp_TINDEX:
-	free(tindex);
-	for (i = 0; i < count; i++)
-	{
-		free(choices_translated[i]);
-		free(choices[i]);
-	}
-	free(choices_translated);
-	free(choices);
+	choices_delete(choices);
 	
 	return ret;
 }
@@ -505,42 +563,27 @@ static int text_handler_multiselect(struct frontend *obj, struct question *q)
  */
 static int text_handler_select(struct frontend *obj, struct question *q)
 {
-	char **choices;
-	char **choices_translated;
+	struct choices *choices = NULL;
 	char answer[10];
-	char *selected;
-	int i, count = 0, choice = 1, def = -1;
-	const char *defval = question_getvalue(q, "");
-	int *tindex = NULL;
+	int i, choice, def = -1;
+	const char *defval;
 	int ret = DC_OK;
-	const char *indices = q_get_indices(obj, q);
 
-	count = strgetargc(q_get_choices_vals(obj, q));
-	if (count <= 0)
+	choices = choices_get(obj, q);
+	if (choices == NULL)
 		return DC_NOTOK;
-	choices = malloc(sizeof(char *) * count);
-	choices_translated = malloc(sizeof(char *) * count);
-	for (i = 0; i < count; ++i)
-	{
-		choices[i] = choices_translated[i] = NULL;
-	}
-	tindex = malloc(sizeof(int) * count);
-	if (strchoicesplitsort(q_get_choices_vals(obj, q), q_get_choices(obj, q), indices, choices, choices_translated, tindex, count) != count)
-	{
-		ret = DC_NOTOK;
-		goto CleanUp_TINDEX;
-	}
 
-	selected = calloc(1, sizeof(char) * count);
-	if (count == 1)
-		defval = choices[0];
+	if (choices->count == 1)
+		defval = choices->choices[0];
+	else
+		defval = question_getvalue(q, "");
 
 	if (defval != NULL)
 	{
-		for (i = 0; i < count; i++)
-			if (strcmp(choices[tindex[i]], defval) == 0)
+		for (i = 0; i < choices->count; i++)
+			if (strcmp(choices->choices[choices->tindex[i]], defval) == 0)
 			{
-				selected[tindex[i]] = 1;
+				choices->selected[choices->tindex[i]] = 1;
 				def = i;
 			}
 	}
@@ -548,8 +591,8 @@ static int text_handler_select(struct frontend *obj, struct question *q)
 	i = 0;
 	choice = -1;
 	do {
-		printlist (obj, q, count, choices_translated, tindex, selected);
-		if (def >= 0 && choices_translated[def]) {
+		printlist (obj, q, choices);
+		if (def >= 0 && choices->choices_translated[def]) {
 			printf(question_get_text(obj, "debconf/text-prompt-default", 
 				"Prompt: '%c' for help, default=%d> "),
 					CHAR_HELP, def+1);
@@ -573,21 +616,11 @@ static int text_handler_select(struct frontend *obj, struct question *q)
 			choice = def;
 		else
 			choice = atoi(answer) - 1;
-	} while (choice < 0 || choice >= count);
-	question_setvalue(q, choices[tindex[choice]]);
+	} while (choice < 0 || choice >= choices->count);
+	question_setvalue(q, choices->choices[choices->tindex[choice]]);
 
   CleanUp_SELECTED:
-	free(selected);
-
-  CleanUp_TINDEX:
-	free(tindex);
-	for (i = 0; i < count; i++) 
-	{
-		free(choices_translated[i]);
-		free(choices[i]);
-	}
-	free(choices_translated);
-	free(choices);
+	choices_delete(choices);
 	
 	return ret;
 }
@@ -787,8 +820,7 @@ static int text_shutdown(struct frontend *obj)
 	struct frontend_data *data = (struct frontend_data *)obj->data;
 	if (NULL != data)
 	{
-		if (NULL != data->previous_title)
-			free(data->previous_title);
+		free(data->previous_title);
 		free(data);
 		obj->data = NULL;
 	}
@@ -900,8 +932,7 @@ static int text_go(struct frontend *obj)
 					underline[underline_len] = '\0';
 					printf("%s\n%s\n\n", obj->title, underline);
 					free(underline);
-					if (data->previous_title)
-						free(data->previous_title);
+					free(data->previous_title);
 					data->previous_title = strdup(obj->title);
 				}
 				text_handler_displaydesc(obj, q);
