@@ -209,12 +209,18 @@ struct template *template_dup(const struct template *t)
         return ret;
 }
 
+#define SAME_STRINGS(a,b) ((a == NULL && b == NULL) || (a != NULL && b != NULL && strcmp(a,b) == 0))
+
 struct template *template_l10nmerge(struct template *ret, const struct template *t)
 {
         struct template_l10n_fields *from, *to;
         int same_choices, same_description;
 
-        if (strcmp(ret->type, t->type) != 0)
+        if (ret->type == NULL)
+                ret->type = STRDUP(t->type);
+        else if (t->type == NULL)
+                return NULL;
+        else if (strcmp(ret->type, t->type) != 0)
                 return NULL;
         if (t->fields == NULL)
                 return ret;
@@ -227,10 +233,9 @@ struct template *template_l10nmerge(struct template *ret, const struct template
 
         from = t->fields;
         to = ret->fields;
-        same_choices = (to->choices == NULL || from->choices == NULL || strcmp(from->choices, to->choices) == 0);
-        same_description = (strcmp(from->description, to->description) == 0 &&
-                            ((from->extended_description == NULL && to->extended_description == NULL) ||
-                             (from->extended_description && to->extended_description && strcmp(from->extended_description, to->extended_description) == 0)));
+        same_choices = SAME_STRINGS(from->choices, to->choices);
+        same_description = SAME_STRINGS(from->description, to->description) &&
+            SAME_STRINGS(from->extended_description, to->extended_description);
         /*  Delete outdated translations */
         if (!same_choices || !same_description)
         {
@@ -833,10 +838,8 @@ struct template *template_load(const char *filename)
 				template_lset(t, lang, "extended_description", extdesc);
 			}
 		}
-		if (lang)
-			free(lang);
-		if (line)
-			free(line);
+		free(lang);
+		free(line);
 	}
 
 	if (t != 0)
